#include <os.h>
#include <math.h>
#include <stdio.h>
#include <ctype.h>
#include <errno.h>


const double mlog10=2.3025850929940457;

static void remove0(char * s){
  int l=strlen(s);
  for (--l;l>0;--l){
    if (s[l]=='0')
      s[l]=0;
    else
      break;
  }
}

char * dtostr(double d,int digits,char * s){
  if (d<0){
    s[0]='-';
    s[1]=0;
    dtostr(-d,digits,s+1);
    return s;
  }
  if (d==0){
    s[0]='0';
    s[1]=0;
    return s;
  }
  int e=ceil(log(d)/mlog10);
  double mult=exp((digits-e)*mlog10);
  if (digits>16)
    digits=16;
  if (e>=0 && e<digits && digits-e<=9){
    int m1=floor(d);
    double dm=d-m1;
    if (dm>0.1){
      dm=dm*mult+.5;
      if (dm>=mult-.5)
	dm -= .5;
      int m2=dm;
      sprintf(s,"%i.%i",m1,m2);
      remove0(s);
      return s;
    }
  }
  d *= exp((digits-e)*mlog10);
  s[0]='0';
  s[1]='.';
  if (digits>9){
    int m1=d/1e9;
    int m2=d-m1*1e9+.5;
    if (e)
      sprintf(s+2,"%i%ie%i",m1,m2,e);
    else {
      sprintf(s+2,"%i%i",m1,m2);
      remove0(s);
    }
  }
  else {
    int m=d+.5;
    if (log(m+1)/mlog10>=digits)
      --m;
    if (e)
      sprintf(s+2,"%ie%i",m,e);
    else {
      sprintf(s+2,"%i",m);
      remove0(s);
    }
  }
  return s;
}

/*
    * strtod.c --
    *
    *   Source code for the "strtod" library procedure.
    *
    * Copyright (c) 1988-1993 The Regents of the University of California.
    * Copyright (c) 1994 Sun Microsystems, Inc.
    *
    * See the file "license.terms" for information on usage and redistribution
    * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
    *
    * RCS: @(#) $Id: strtod.c,v 1.1.1.4 2003/03/06 00:09:04 landonf Exp $
    */

    static int maxExponent = 511;   /* Largest possible base 10 exponent.  Any
                 * exponent larger than this will already
                 * produce underflow or overflow, so there's
                 * no need to worry about additional digits.
                 */
    static double powersOf10[] = {   /* Table giving binary powers of 10.  Entry */
        10.,         /* is 10^2^i.  Used to convert decimal */
        100.,         /* exponents into floating-point numbers. */
        1.0e4,
        1.0e8,
        1.0e16,
        1.0e32,
        1.0e64,
        1.0e128,
        1.0e256
    };

    /*
    *----------------------------------------------------------------------
    *
    * strtod --
    *
    *   This procedure converts a floating-point number from an ASCII
    *   decimal representation to internal double-precision format.
    *
    * Results:
    *   The return value is the double-precision floating-point
    *   representation of the characters in string.  If endPtr isn't
    *   NULL, then *endPtr is filled in with the address of the
    *   next character after the last one that was part of the
    *   floating-point number.
    *
    * Side effects:
    *   None.
    *
    *----------------------------------------------------------------------
    */

    double
    Strtod(const char * string, char ** endPtr)
    /* A decimal ASCII floating-point number,
    * optionally preceded by white space.
    * Must have form "-I.FE-X", where I is the
    * integer part of the mantissa, F is the
    * fractional part of the mantissa, and X
    * is the exponent.  Either of the signs
    * may be "+", "-", or omitted.  Either I
    * or F may be omitted, or both.  The decimal
    * point isn't necessary unless F is present.
    * The "E" may actually be an "e".  E and X
    * may both be omitted (but not just one).
    */
    /* If non-NULL, store terminating character's
                 * address here. */
    {
        int sign, expSign = 0;
        double fraction, dblExp, *d;
        register const char *p;
        register int c;
        int exp = 0;      /* Exponent read from "EX" field. */
        int fracExp = 0;      /* Exponent that derives from the fractional
                 * part.  Under normal circumstatnces, it is
                 * the negative of the number of digits in F.
                 * However, if I is very long, the last digits
                 * of I get dropped (otherwise a long I with a
                 * large negative exponent could cause an
                 * unnecessary overflow on I alone).  In this
                 * case, fracExp is incremented one for each
                 * dropped digit. */
        int mantSize;      /* Number of digits in mantissa. */
        int decPt;         /* Number of mantissa digits BEFORE decimal
                 * point. */
        const char *pExp;      /* Temporarily holds location of exponent
                 * in string. */

        /*
         * Strip off leading blanks and check for a sign.
         */

        p = string;
        while (isspace((unsigned char)*p)) {
       p += 1;
        }
        if (*p == '-') {
       sign = 1;
       p += 1;
        } else {
       if (*p == '+') {
           p += 1;
       }
       sign = 0;
        }

        /*
         * Count the number of digits in the mantissa (including the decimal
         * point), and also locate the decimal point.
         */

        decPt = -1;
        for (mantSize = 0; ; mantSize += 1)
        {
       c = *p;
       if (!isdigit(c)) {
           if ((c != '.') || (decPt >= 0)) {
          break;
           }
           decPt = mantSize;
       }
       p += 1;
        }

        /*
         * Now suck up the digits in the mantissa.  Use two integers to
         * collect 9 digits each (this is faster than using floating-point).
         * If the mantissa has more than 18 digits, ignore the extras, since
         * they can't affect the value anyway.
         */
       
        pExp  = p;
        p -= mantSize;
        if (decPt < 0) {
       decPt = mantSize;
        } else {
       mantSize -= 1;         /* One of the digits was the point. */
        }
        if (mantSize > 18) {
       fracExp = decPt - 18;
       mantSize = 18;
        } else {
       fracExp = decPt - mantSize;
        }
        if (mantSize == 0) {
       fraction = 0.0;
       p = string;
       goto done;
        } else {
       int frac1, frac2;
       frac1 = 0;
       for ( ; mantSize > 9; mantSize -= 1)
       {
           c = *p;
           p += 1;
           if (c == '.') {
          c = *p;
          p += 1;
           }
           frac1 = 10*frac1 + (c - '0');
       }
       frac2 = 0;
       for (; mantSize > 0; mantSize -= 1)
       {
           c = *p;
           p += 1;
           if (c == '.') {
          c = *p;
          p += 1;
           }
           frac2 = 10*frac2 + (c - '0');
       }
       fraction = (1.0e9 * frac1) + frac2;
        }

        /*
         * Skim off the exponent.
         */

        p = pExp;
        if ((*p == 'E') || (*p == 'e')) {
       p += 1;
       if (*p == '-') {
           expSign = 1;
           p += 1;
       } else {
           if (*p == '+') {
          p += 1;
           }
           expSign = 0;
       }
       if (!isdigit((unsigned char)*p)) {
           p = pExp;
           goto done;
       }
       while (isdigit((unsigned char)*p)) {
           exp = exp * 10 + (*p - '0');
           p += 1;
       }
        }
        if (expSign) {
       exp = fracExp - exp;
        } else {
       exp = fracExp + exp;
        }

        /*
         * Generate a floating-point number that represents the exponent.
         * Do this by processing the exponent one bit at a time to combine
         * many powers of 2 of 10. Then combine the exponent with the
         * fraction.
         */
       
        if (exp < 0) {
       expSign = 1;
       exp = -exp;
        } else {
       expSign = 0;
        }
        if (exp > maxExponent) {
       exp = maxExponent;
       errno = ERANGE;
        }
        dblExp = 1.0;
        for (d = powersOf10; exp != 0; exp >>= 1, d += 1) {
       if (exp & 01) {
           dblExp *= *d;
       }
        }
        if (expSign) {
       fraction /= dblExp;
        } else {
       fraction *= dblExp;
        }

    done:
        if (endPtr != 0) {
       *endPtr = (char *) p;
        }

        if (sign) {
       return -fraction;
        }
        return fraction;
    }



